#:coding=utf-8:

from django.http import HttpRequest
from django.test import TestCase as DjangoTestCase


class ClassesTestCase(DjangoTestCase):
    def test_simple(self):
        response = self.client.get("/test")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"OK")


class RenderToTestCase(DjangoTestCase):

    def test_render_to(self):
        from testapp.views import myview
        resp = myview(HttpRequest())
        self.assertEqual(resp.content, b'<html><body>MY VALUE</body></html>\n')

    def test_render_to_httpresponse(self):
        from testapp.views import myview2
        resp = myview2(HttpRequest())
        self.assertEqual(resp.content, b'Error!')


class AjaxResponseTestCase(DjangoTestCase):

    def test_ajax_view(self):
        from testapp.views import my_ajax_view
        resp = my_ajax_view(HttpRequest())
        self.assertEqual(resp["content-type"], 'application/json')
        self.assertEqual(resp.content, b'{"my_value": "MY VALUE"}')

    def test_ajax_view_httpresponse(self):
        from testapp.views import my_ajax_view2
        resp = my_ajax_view2(HttpRequest())
        self.assertEqual(resp["content-type"], 'text/html; charset=utf-8')
        self.assertEqual(resp.content, b'Error!')



#:coding=utf-8:
from datetime import datetime

import pytest
import factory
from django.db import connection
from django.core.exceptions import ValidationError
from django.db.models import IntegerField, BigIntegerField

from models.fields.models import (
    BigIntModel,
    BigToSmallModel,
    ManyToManyModel,
    BigIDModel,
    SmallIDModel,
)


class BigIdModelFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = BigIDModel

    id = factory.Sequence(lambda n: n * 10000000 + 1)


class SmallIDModelFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = SmallIDModel


class BigIntModelFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = BigIntModel

    big_id_obj = factory.SubFactory(BigIdModelFactory)


class BigToSmallModelFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = BigToSmallModel

    small_id_obj = factory.SubFactory(SmallIDModelFactory)


class ManyToManyModelFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ManyToManyModel
        skip_postgeneration_save = True

    @factory.post_generation
    def bigids(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of groups were passed in, use them
            for bigid in extracted:
                self.bigids.add(bigid)


@pytest.mark.django_db
class TestBigForeignKey(object):

    @pytest.fixture
    def make_bigid(self):
        return BigIdModelFactory.create

    @pytest.fixture
    def make_bigint(self):
        return BigIntModelFactory.create

    @pytest.fixture
    def make_bigtosmall(self):
        return BigToSmallModelFactory.create

    @pytest.fixture
    def make_manytomany(self):
        return ManyToManyModelFactory.create

    def test_simple(self, make_bigint):
        make_bigint()
        make_bigint()

        qs = BigIntModel.objects.recently_updated()

        # クエリがデータを返すかどうかをチェック
        assert qs

        for obj in qs:
            assert obj.big_id_obj_id

    def test_bigtosmall(self, make_bigtosmall):
        make_bigtosmall()
        make_bigtosmall()

        qs = BigToSmallModel.objects.recently_updated()

        # クエリがデータを返すかどうかをチェック
        assert qs

        for obj in qs:
            assert obj.small_id_obj_id

    def test_bigforeignkey_db_type(self, make_bigint, make_bigtosmall):
        make_bigint()
        make_bigint()
        make_bigtosmall()
        make_bigtosmall()

        qs = BigIntModel.objects.recently_updated()
        # クエリがデータを返すかどうかをチェック
        assert qs
        for obj in qs:
            for f in obj._meta.fields:
                if f.name == "big_id_obj":
                    db_type = f.db_type(connection)
                    bigint_type = BigIntegerField().db_type(connection=connection)
                    assert db_type == bigint_type  # oracleでは動かない

        qs = BigToSmallModel.objects.recently_updated()
        # クエリがデータを返すかどうかをチェック
        assert qs
        for obj in qs:
            for f in obj._meta.fields:
                if f.name == "small_id_obj":
                    db_type = f.db_type(connection)
                    int_type = IntegerField().db_type(connection=connection)
                    assert db_type == int_type  # oracleでは動かない

    def test_manytomany_db_type(self, make_manytomany, make_bigid):
        b1 = make_bigid()
        b2 = make_bigid()
        b3 = make_bigid()
        make_manytomany(bigids=(b1, b2))
        make_manytomany(bigids=[b1])
        make_manytomany(bigids=[b1, b2, b3])

        for obj in ManyToManyModel.objects.all():
            bigids = obj.bigids.all()
            assert len(bigids) > 0
            for bigid in bigids:
                assert bigid.id


@pytest.mark.django_db
class TestBadBigAutoId(object):
    def test_bad_id(self):
        with pytest.raises(ValidationError):
            BigIDModel._meta.pk.to_python("bad id")




import os
import warnings
warnings.filterwarnings("error", module='django')
warnings.filterwarnings("error", module='beproud.django.commons')

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'


from django.apps import apps
apps.populate([
    'testapp',
    'shortcuts_app',
    'beproud.django.commons',
    'models.base',
    'models.fields',
])



from datetime import datetime

import pytest
import factory

from models.base.models import BaseModel


__all__ = (
    'TestCompareModel',
    'TestCopyModel',
)


class BaseModelFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = BaseModel


@pytest.mark.django_db
class TestCompareModel(object):

    @pytest.fixture
    def target(self):
        from beproud.django.commons.models.utils import compare_obj
        return compare_obj

    def test_simple(self, target):
        base_obj = BaseModelFactory.create(del_flg=False)
        new_obj = BaseModelFactory.create(del_flg=True)
        result = target(base_obj, new_obj)

        del_flg_field = BaseModel._meta.get_field("del_flg")
        assert result[del_flg_field] == (False, True)


@pytest.mark.django_db
class TestCopyModel(object):

    @pytest.fixture
    def target(self):
        from beproud.django.commons.models.utils import copy_obj
        return copy_obj

    def test_simple(self, target):
        from_obj = BaseModelFactory.create(utime=datetime(2020, 4, 20))
        to_obj = BaseModel()
        target(from_obj, to_obj)

        assert from_obj.id != to_obj.id
        assert from_obj.ctime == to_obj.ctime
        assert from_obj.utime == to_obj.utime
        assert from_obj.del_flg is to_obj.del_flg



#:coding=utf-8:

from django.test import TestCase as DjangoTestCase

from django.forms import Form

from beproud.django.commons.forms import EmailField

__all__ = (
    'EmailFieldTest',
    'JSONFormFieldTest',
    'JSONWidgetTest',
)


class EmailTestForm(Form):
    email = EmailField(label="email")


class EmailFieldTest(DjangoTestCase):

    def test_basic_email(self):
        form = EmailTestForm({"email": "spam@eggs.com"})
        self.assertTrue(form.is_valid())

    def test_keitai_email(self):
        form = EmailTestForm({"email": "-spam..eggs-@softbank.ne.jp"})
        self.assertTrue(form.is_valid())

        form = EmailTestForm({"email": ".*&$.-spam..!!eggs!!-.*.@ezweb.ne.jp"})
        self.assertTrue(form.is_valid())

    def test_plus_email(self):
        form = EmailTestForm({"email": "spam+extra@eggs.com"})
        self.assertTrue(form.is_valid())

    def test_multi_email(self):
        form = EmailTestForm({"email": "aaa spam+extra@eggs.com email@email.com"})
        self.assertFalse(form.is_valid())

    def test_longtld(self):
        form = EmailTestForm({"email": "spam@eggs.engineer"})
        self.assertTrue(form.is_valid())

    def test_punycode(self):
        form = EmailTestForm({"email": "spam@eggs.xn--i1b6b1a6a2e"})
        self.assertTrue(form.is_valid())





#:coding=utf-8:

from django.db import models

from beproud.django.commons.models import (
    BaseModel,
    BigAutoField,
)


class BigIDModel(BaseModel):
    id = BigAutoField(primary_key=True)
    class Meta:
        app_label = 'fields'


class BigIntModel(BaseModel):
    big_id_obj = models.ForeignKey(BigIDModel, on_delete=models.CASCADE)
    class Meta:
        app_label = 'fields'


class SmallIDModel(BaseModel):
    class Meta:
        app_label = 'fields'


class BigToSmallModel(BaseModel):
    small_id_obj = models.ForeignKey(SmallIDModel, on_delete=models.CASCADE)
    class Meta:
        app_label = 'fields'


class ManyToManyModel(BaseModel):
    bigids = models.ManyToManyField(BigIDModel)
    class Meta:
        app_label = 'fields'



#:coding=utf-8:

from django.urls import path
from django.http import HttpResponse

from beproud.django.commons.views.decorators import render_to, ajax_request
from beproud.django.commons.views import Views


class TestViews(Views):
    def test(self, request):
        return HttpResponse("OK")

    def get_urls(self):
        urls = super(TestViews, self).get_urls()
        my_urls = [
            path(
                'test',
                self.test,
                name='testview_test',
            ),
        ]
        return my_urls + urls


@render_to("view_tests/render_to.html")
def myview(request):
    return {"my_value": "MY VALUE"}


@render_to("view_tests/render_to.html")
def myview2(request):
    return HttpResponse("Error!")


@ajax_request
def my_ajax_view(request):
    return {'my_value': 'MY VALUE'}


@ajax_request
def my_ajax_view2(request):
    return HttpResponse("Error!")



#:coding=utf-8:

from beproud.django.commons.models import (
    DatedModel as DatedModelBase,
    BaseModel as BaseModelBase
)


class DatedModel(DatedModelBase):
    class Meta:
        app_label = 'base'


class BaseModel(BaseModelBase):
    class Meta:
        app_label = 'base'



#:coding=utf-8:

from bs4 import BeautifulSoup

from django.test import TestCase as DjangoTestCase
from django.template.loader import render_to_string

__all__ = (
    'StringTagsTestCase',
    'HtmlTagsTestCase',
)


class StringTagsTestCase(DjangoTestCase):

    def test_abbrev(self):
        output = render_to_string("templatetags_tests/string_tags.html",
                                  {"my_data": "1234567890abcdefghi"})
        self.assertEqual(output, "<html><body>1234567...</body></html>\n")

    def test_abbrev_filter_tag(self):
        output = render_to_string("templatetags_tests/string_tags2.html",
                                  {"my_data": "1234567890abcdefghi"})
        self.assertEqual(output, "<html><body>1234567...</body></html>\n")


class HtmlTagsTestCase(DjangoTestCase):

    def test_to_anchor(self):
        output = render_to_string("templatetags_tests/to_anchor.html", {
            "my_data": u"これはテストデータ。http://www.beproud.jp/これはテストデータ。",
        })

        soup = BeautifulSoup(output, "html.parser")
        anchors = soup.findAll('a')
        self.assertTrue(len(anchors), 1)
        self.assertEqual(anchors[0]['target'], '_blank')
        self.assertEqual(anchors[0]['rel'], ['nofollow'])
        self.assertEqual(anchors[0].contents[0], 'http://www.beproud.jp/')

    def test_to_anchortrunc(self):
        output = render_to_string("templatetags_tests/to_anchortrunc.html", {
            "my_data": u"これはテストデータ。http://www.beproud.jp/これはテストデータ。",
        })

        soup = BeautifulSoup(output, "html.parser")
        anchors = soup.findAll('a')
        self.assertTrue(len(anchors), 1)
        self.assertEqual(anchors[0]['target'], '_blank')
        self.assertEqual(anchors[0]['rel'], ['nofollow'])
        self.assertEqual(anchors[0].contents[0], 'http://www.b...')



# -*- coding: utf-8 -*-
import os

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'beproud.django.commons',
    'testapp',
    'shortcuts_app',
    'models.base',
    'models.fields',
)
MIDDLEWARE = []  # to pass deprecation tcheck
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}
ROOT_URLCONF = 'testapp.urls'

TEMPLATES=[{
    'DIRS': [os.path.join(os.path.dirname(__file__), 'templates')],
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
}]
SECRET_KEY = '<key>'
USE_TZ = False



# -*- coding: utf-8 -*-

from datetime import date
from beproud.django.commons import VERSION


source_suffix = '.rst'
master_doc = 'index'
project = u'bpcommons'
copyright = u'{:%Y}, BeProud Inc.'.format(date.today())

version = release = VERSION

language = 'ja'

pygments_style = 'sphinx'

html_theme='classic'




#:coding=utf-8:

import pytest
from django.test import TestCase as DjangoTestCase

from beproud.django.commons.shortcuts import get_object_or_None, make_simple_response


@pytest.mark.django_db
class TestGetObjectOrNone(object):

    @pytest.fixture
    def target(self):
        from shortcuts_app.models import ShortcutModel
        return ShortcutModel

    def test_simple(self, target):
        instance = target.objects.create(name='test')
        obj = get_object_or_None(target, pk=instance.pk)
        assert obj is not None

    def test_queryset(self, target):
        i1 = target.objects.create(name='queryset')
        i2 = target.objects.create(name='queryset')
        i3 = target.objects.create(name='queryset')

        qs = target.objects.filter(name="queryset")
        obj = get_object_or_None(qs, pk=i2.pk)
        assert obj is not None

    def test_none(self, target):
        i1 = target.objects.create(name='test')
        i2 = target.objects.create(name='test')
        pk = max(i1.pk, i2.pk) + 1

        obj = get_object_or_None(target, pk=pk)
        assert obj is None


class MakeSimpleResponseTestCase(DjangoTestCase):

    def test_simple(self):
        response = make_simple_response()
        self.assertEqual(
            response.content,
            b'{"msg": "\\u51e6\\u7406\\u304c\\u6210\\u529f\\u3057\\u307e\\u3057\\u305f"}',
        )



#:coding=utf-8:
from datetime import datetime

import pytest
import factory

from models.base.models import DatedModel, BaseModel


class DatedModelFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = DatedModel


class BaseModelFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = BaseModel


@pytest.mark.django_db
class TestDatedModel(object):

    @pytest.fixture
    def create_target(self):
        return DatedModelFactory.create

    def test_simple(self, create_target):
        create_target(utime=datetime(2016, 10, 1))
        create_target(utime=datetime(2016, 10, 3))
        create_target(utime=datetime(2016, 10, 2))

        qs = DatedModel.objects.recently_updated()

        # クエリがデータを返すかどうかをチェック
        assert qs

        previous = None
        for obj in qs:
            if previous:
                assert obj.utime <= previous.utime
            previous = obj


@pytest.mark.django_db
class TestBaseModel(object):

    @pytest.fixture
    def create_target(self):
        return BaseModelFactory.create

    def test_be(self, create_target):
        create_target()
        create_target()
        create_target(del_flg=True)

        qs = BaseModel.objects.be()

        for obj in qs:
            assert not obj.del_flg

    def test_remove(self, create_target):
        obj = create_target()
        assert not obj.del_flg

        obj.remove()
        assert obj.del_flg

        obj_updated = BaseModel.objects.get(pk=1)
        assert obj_updated.del_flg

    def test_unremove(self, create_target):
        obj = create_target(del_flg=True)
        assert obj.del_flg

        obj.unremove()
        assert not obj.del_flg

        obj_updated = BaseModel.objects.get(pk=obj.pk)
        assert not obj_updated.del_flg
